// DBGui2Dlg.cpp : implementation file

#include "stdafx.h"
#include "DBGui2.h"
#include "DBGui2Dlg.h"
#include ".\dbgui2dlg.h"
#include "Serial.h"
#include "afxmt.h"
#include "assert.h"
#include <fstream>
 
 

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define DISABLE_MODE 0
#define CHARGE_MODE 1
#define DISCHARGE_MODE 2
 
#define MAX_FILE_SIZE          4096
#define PACKET_SIZE            256
#define CRC_OK                 0x01
#define REPROGRAM_COMMAND      0x04
#define CRC_ACK                0x01
#define CRC_NACK               0x07
#define REPROG_DONE            0x14 

#define NET_ADDR_ADCNS         0x11
#define NET_ADDR_POWER         0x44
#define NET_ADDR_CAMERA        0x22

#define DB_CMD_REPROG_MCU      0x01     //reprogram subsystems
#define DB_CMD_GET_ALL_DATA    0x02     //PC request ADC data from DB
#define DB_CMD_GET_CUCP        0x03     //PC request data from MCU
#define DB_CMD_BAT_DISCHRG     0x04     //start charge circuit
#define DB_CMD_BAT_CHARGE      0x05     //stop charge circuit
#define DB_CMD_BAT_OFF         0x06
#define DB_CMD_INHIBIT_POS     0x07
#define DB_CMD_INHIBIT_NEG     0x08
#define DB_CMD_INHIBIT_OFF     0x09

struct ALL_MCU_DATA {
	char CHARGE_CURRENT_SENSE;
	char DISCHARGE_CURRENT_SENSE;
	char VBUS_CURRENT_SENSE;
	char inhibitState;
	char batteryState;
};

struct ALL_MCU_DATA allMCUData;

CListBox *c_ADCList;
CEdit *c_CUCP;
CSerial serial;
CSemaphore* serialSem;
extern CDBGui2Dlg *myDlg;
float lastDischargeMA[5] = {0.0, 0.0, 0.0, 0.0, 0.0};

void CDBGui2Dlg::enableAllButtons() {
    this->m_btn_batt_charge.EnableWindow(1);
	this->m_btn_batt_discharge.EnableWindow(1);
	this->m_btn_batt_off.EnableWindow(1);
	this->m_btn_inhibit_neg.EnableWindow(1);
	this->m_btn_inhibit_off.EnableWindow(1);
	this->m_btn_inhibit_pos.EnableWindow(1);
	this->m_btn_reprogram.EnableWindow(1);
}

void CDBGui2Dlg::disableAllButtons() {
    this->m_btn_batt_charge.EnableWindow(0);
	this->m_btn_batt_discharge.EnableWindow(0);
	this->m_btn_batt_off.EnableWindow(0);
	this->m_btn_inhibit_neg.EnableWindow(0);
	this->m_btn_inhibit_off.EnableWindow(0);
	this->m_btn_inhibit_pos.EnableWindow(0);
	this->m_btn_reprogram.EnableWindow(0);
}

static void lockSerial() {
	serialSem->Lock();
	myDlg->disableAllButtons();
}

static void unlockSerial() {
	serialSem->Unlock();
	Sleep(500);
	myDlg->enableAllButtons();
}

static inline char waitSerialByte() {
	char recBuffer;
	char numBytesRead;
	do {
		numBytesRead = serial.ReadData(&recBuffer, 1);
	} while (numBytesRead == 0);
	return recBuffer;
}

UINT GetStatusThread(LPVOID params) {
	char sendBuffer;
	char recBuffer[50];
	int numIncomingBytes;
	
	while (1) {
		Sleep(200);
		
		serialSem->Lock();	
		sendBuffer = DB_CMD_GET_ALL_DATA;
		serial.SendData(&sendBuffer, 1);
		recBuffer[0] = waitSerialByte();
		if (recBuffer[0] == DB_CMD_GET_ALL_DATA) {
			serial.ReadData(recBuffer, sizeof(struct ALL_MCU_DATA));

			// update list!
			for (int i=0; i<5; i++)
				c_ADCList->DeleteString(0);
			
            CString str;
			str.Format("%d", recBuffer[0]);		c_ADCList->AddString(str);
						
			// discharge current
			// move back filter
			for (i=0; i<4; i++)
				lastDischargeMA[i] = lastDischargeMA[i+1];
			lastDischargeMA[4] = (1000.0*recBuffer[1])/127;

			float avg = 0.0;

			for (i=0; i<5; i++)
				avg += lastDischargeMA[i];

			str.Format("%5.1f", avg/5.0);
			c_ADCList->AddString(str);

			str.Format("%d", recBuffer[2]);		c_ADCList->AddString(str);

			switch (recBuffer[3]) {
				case 10:
					c_ADCList->AddString("+");
					break;
				case 11:
					c_ADCList->AddString("-");
					break;
				case 12:
					c_ADCList->AddString("X");
					break;
				default:
					c_ADCList->AddString("-");
					break;
			}

			switch (recBuffer[4]) {
				case 7:
					c_ADCList->AddString("C");
					break;
				case 8:
					c_ADCList->AddString("D");
					break;
				case 9:
					c_ADCList->AddString("X");
					break;
				default:
					c_ADCList->AddString("-");
					break;
			}
		}

		sendBuffer = DB_CMD_GET_CUCP;
		serial.SendData(&sendBuffer, 1);
		recBuffer[0] = waitSerialByte();
		if (recBuffer[0] == DB_CMD_GET_CUCP) {
			numIncomingBytes = waitSerialByte(); // number of incoming bytes
			for (int i=0; i<numIncomingBytes; i++) {
				CString str, currStr;
				c_CUCP->GetWindowText(currStr);
				str.Format("%X\r\n", waitSerialByte());
				str.Append(currStr);
				c_CUCP->SetWindowText(str);
			}
		}
		serialSem->Unlock();	
	}
	return 0;
}

CString friendlyChargeMode(int cm) {
	if (cm == DISABLE_MODE) return "Disable";
	else if (cm == CHARGE_MODE) return "Charging";
	else if (cm == DISCHARGE_MODE) return "Discharging";
}

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// CDBGui2Dlg dialog



CDBGui2Dlg::CDBGui2Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDBGui2Dlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDBGui2Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO3, c_COMBO3);
	DDX_Control(pDX, IDC_EDIT2, c_EDIT2);
	DDX_Control(pDX, IDC_EDIT4, c_EDIT4);
	DDX_Control(pDX, IDC_LIST10, c_LIST10);
	DDX_Control(pDX, IDC_BUTTON2, c_BUTTON2);
	DDX_Control(pDX, IDC_LIST1, c_LIST1);
	DDX_Control(pDX, IDC_LIST2, c_LIST2);
	DDX_Control(pDX, IDC_LIST3, c_LIST3);
	DDX_Control(pDX, IDC_EDIT3, C_EDIT3);
	DDX_Control(pDX, IDC_EDIT5, C_EDIT5);
	DDX_Control(pDX, IDOK2, m_btn_reprogram);
	DDX_Control(pDX, IDC_BUTTON1, m_btn_batt_charge);
	DDX_Control(pDX, IDC_BUTTON3, m_btn_batt_discharge);
	DDX_Control(pDX, IDC_BUTTON4, m_btn_batt_off);
	DDX_Control(pDX, IDC_BUTTON5, m_btn_inhibit_pos);
	DDX_Control(pDX, IDC_BUTTON6, m_btn_inhibit_neg);
	DDX_Control(pDX, IDC_BUTTON7, m_btn_inhibit_off);
}

BEGIN_MESSAGE_MAP(CDBGui2Dlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_BUTTON2, OnBnClickedButton2)
	ON_BN_CLICKED(IDOK2, OnBnClickedOk2)
	ON_BN_CLICKED(IDC_BUTTON1, OnBnClickedButton1)
	ON_BN_CLICKED(IDC_BUTTON3, OnBnClickedButton3)
	ON_BN_CLICKED(IDC_BUTTON4, OnBnClickedButton4)
	ON_BN_CLICKED(IDC_BUTTON5, OnBnClickedButton5)
	ON_BN_CLICKED(IDC_BUTTON6, OnBnClickedButton6)
	ON_BN_CLICKED(IDC_BUTTON7, OnBnClickedButton7)
	ON_CBN_SELCHANGE(IDC_COMBO3, OnCbnSelchangeCombo3)
END_MESSAGE_MAP()


// CDBGui2Dlg message handlers

BOOL CDBGui2Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	HICON icon = AfxGetApp()->LoadIcon(IDI_ICON2);
	c_BUTTON2.SetIcon(icon);

	c_LIST1.AddString("Charging Current");
	c_LIST1.AddString("Discharging Current");
	c_LIST1.AddString("CUSat Bus Current");
	c_LIST1.AddString("Inhibit State");
	c_LIST1.AddString("Battery Mode");

	c_LIST2.AddString("A");
	c_LIST2.AddString("mA");
	c_LIST2.AddString("A");
	c_LIST2.AddString("-");
	c_LIST2.AddString("-");

	c_LIST3.AddString("-");
	c_LIST3.AddString("-");
	c_LIST3.AddString("-");
	c_LIST3.AddString("X");
	c_LIST3.AddString("X");

	c_ADCList = &c_LIST3;
	c_CUCP = &c_EDIT2;

	// setup serial port semaphore
	serialSem = new CSemaphore();

	// setup serial port
	if (!serial.Open(1, 9600)) {
		MessageBox("ERROR can't open serial port!");
		exit(0);
	} else {
		//MessageBox("OPEN");
	}
	
	// start thread to get values every time period
	AfxBeginThread(GetStatusThread,NULL,THREAD_PRIORITY_NORMAL,0,0,NULL);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CDBGui2Dlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDBGui2Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDBGui2Dlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CDBGui2Dlg::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here
	OnCancel();
}

void CDBGui2Dlg::OnLbnSelchangeList3()
{
	// TODO: Add your control notification handler code here
}

void CDBGui2Dlg::OnBnClickedOk()   //display output text?
{
//	OnOK();
}
void CDBGui2Dlg::OnBnClickedButton2()
{
  //char c[10];
  //int f_Size,numPackets;;
  //CString MFCString;
 

  CFileDialog FileDialog(TRUE,
                         "*.*",
                         NULL,
                         OFN_HIDEREADONLY,
                         "All Files||");

  if(FileDialog.DoModal() == IDOK)
  {
    PathName = FileDialog.GetPathName();
	c_EDIT4.SetWindowText(PathName);
	//f_Size = FileSize(PathName);
	//numPackets = f_Size/PACKET_SIZE;
	//if (f_Size%PACKET_SIZE) numPackets++;
	//itoa(numPackets,c,10);
	//out_put += "File Name: " +FileDialog.GetFileName()+ "   Num Packets: " + c +"\r\n" ;
    //c_EDIT2.SetWindowText(out_put);
   }	
}

void CDBGui2Dlg::OnBnClickedOk2()   //reprogram button
{  
 
	char buffer[MAX_FILE_SIZE+1] = {0}; // max 4k filesize
	char sendBuffer[256];
	CString cs;
	int fileSize;
 	int i;
	char me;

 	// read file into buffer, note rest of file will get 0xFF padding
	serialSem->Lock();
	std::ifstream f;
	f.open(PathName, std::ios_base::binary | std::ios_base::in);
	assert (f != NULL);
	f.read(buffer, MAX_FILE_SIZE);
	f.close(); 
 
	
	// get file size
	fileSize = FileSize(PathName); 
    c_EDIT2.SetWindowText("get file size");
	cs.Format("File Size :  %d bytes\r\n", fileSize);
	out_put += cs;
    c_EDIT2.SetWindowText(out_put);

	// pad buffer w/ 0xFF
	for (i=fileSize; i<MAX_FILE_SIZE; i++)
		buffer[i] = 0xFF;
 
	// find number of packets to send
	int numPackets = fileSize/PACKET_SIZE;
	if (fileSize%PACKET_SIZE != 0) numPackets++;
	cs.Format("Num Packets = %d\r\n", numPackets);
    out_put += cs;
    c_EDIT2.SetWindowText(out_put);


	sendBuffer[0] = DB_CMD_REPROG_MCU;
	serial.SendData(sendBuffer,1);
    out_put +=  "Sent command\r\n";
	c_EDIT2.SetWindowText(out_put);
	me = waitSerialByte();
	if (me != DB_CMD_REPROG_MCU) {
		AfxMessageBox("Can't reprogram Error 1");
	}
	out_put += "Command echo from DB Received. \r\n";
	c_EDIT2.SetWindowText(out_put);

	// first send address, wait for echo
	sendBuffer[0] = net_addr;
	serial.SendData(sendBuffer, 1);
	out_put +=  "Sent addresss\r\n";
	c_EDIT2.SetWindowText(out_put);
	me = waitSerialByte();
	if (me != net_addr) {
		AfxMessageBox("Can't reprogram Error 2");
	}

    out_put += "Addr echo from MCU received. \r\n";
	c_EDIT2.SetWindowText(out_put);

 
	// send command, wait for echo
	sendBuffer[0] = REPROGRAM_COMMAND;
	serial.SendData(sendBuffer, 1);
	assert(waitSerialByte() == REPROGRAM_COMMAND);
    out_put += "Command echo for MCU received. \r\n"; 
	c_EDIT2.SetWindowText(out_put);

	// send num packets
	sendBuffer[0] = numPackets / 256;
	sendBuffer[1] = numPackets % 256;
	serial.SendData(sendBuffer, 2);
	
	// now send PACKET_SIZE bytes at a time, w/ checksum
	int currPacket = 0;
	while (currPacket < numPackets) {
		// send packet
		Sleep(800);
		for (int i=0; i<PACKET_SIZE; i++) {
			sendBuffer[i] = buffer[(currPacket*PACKET_SIZE) + i];
		}
		serial.SendData(sendBuffer, PACKET_SIZE);

		// send crc55
		unsigned short crc = 0;
		crc |= calc_crc(sendBuffer, PACKET_SIZE);
		sendBuffer[0] = crc / 256;
		sendBuffer[1] = crc % 256;
	-	serial.SendData(sendBuffer, 2);

        cs.Format("Send packet %d ",currPacket);
		out_put += cs;
        c_EDIT2.SetWindowText(out_put);
		// wait for the OK from slave
		// printf("waiting on response of chk[%d] = %X ", currPacket, crc);
		char c = waitSerialByte();
		if (c == CRC_ACK) {
			//printf(" OK!");
			out_put += "CRC_ACK Received. \r\n"; 
	        c_EDIT2.SetWindowText(out_put);
			currPacket++;
		}
	//	printf("\n");
	}
	
	assert(waitSerialByte() == REPROG_DONE);
    out_put += "Program updated.\r\n";
    c_EDIT2.SetWindowText(out_put);   
	serialSem->Unlock();
}

// generic command send. returns 0 if failure, 1 otherwise
static int SendCommand(char cmd) {
	char c = cmd;
	char recBuffer;
	lockSerial();
	serial.SendData(&c, 1);
	recBuffer = waitSerialByte();
	unlockSerial();
	return (recBuffer == cmd);
}

// battery charge
void CDBGui2Dlg::OnBnClickedButton1()
{
	if (!SendCommand(DB_CMD_BAT_CHARGE))
		AfxMessageBox("Error charging batteries!");	
}

// discharge batteries
void CDBGui2Dlg::OnBnClickedButton3()
{
	if (!SendCommand(DB_CMD_BAT_DISCHRG))
		AfxMessageBox("Error discharging batteries!");
}

// batteries off
void CDBGui2Dlg::OnBnClickedButton4()
{	
	if (!SendCommand(DB_CMD_BAT_OFF))
		AfxMessageBox("Error turning battery charging off!");
}

// +12 inhibit
void CDBGui2Dlg::OnBnClickedButton5()
{
	if (!SendCommand(DB_CMD_INHIBIT_POS))
		AfxMessageBox("Error setting +12 inhibit!");
}

// -12 inhibit
void CDBGui2Dlg::OnBnClickedButton6()
{
	if (!SendCommand(DB_CMD_INHIBIT_NEG))
		AfxMessageBox("Error setting -12 inhibit!");
}

// inhibit off
void CDBGui2Dlg::OnBnClickedButton7()
{
	if (!SendCommand(DB_CMD_INHIBIT_OFF))
		AfxMessageBox("Error disabling inhibit!");
}

unsigned short CDBGui2Dlg::calc_crc(char *ptr, int count)
{
    unsigned short crc = 0; 
    char i; 
     
    while (--count >= 0) { 
        crc = crc ^ (int) *(ptr++) << 8; 
        i = 8; 
        do { 
			if (crc & 0x8000) 
	            crc = crc << 1 ^ 0x1021; 
			else 
	            crc = crc << 1; 
		} while(--i); 
    }
    return crc;

}

int CDBGui2Dlg::FileSize( const char * szFileName ) 
 
{
  std::ifstream f;
  f.open(szFileName, std::ios_base::binary | std::ios_base::in);
  if (!f.good() || f.eof() || !f.is_open()) { return 0; }
  f.seekg(0, std::ios_base::beg);
  std::ifstream::pos_type begin_pos = f.tellg();
  f.seekg(0, std::ios_base::end);
  return static_cast<int>(f.tellg() - begin_pos);

}
void CDBGui2Dlg::OnCbnSelchangeCombo3()
{
		 
		int idx = c_COMBO3.GetCurSel();
		if( idx < 0 ) return;
		if(idx ==0) { out_put += "You selected ADCNS\r\n";      net_addr = NET_ADDR_ADCNS;}
		if(idx ==2) { out_put += "You selected POWER\r\n";      net_addr = NET_ADDR_POWER;}
		if(idx ==1) { out_put += "You selected CAMERA\r\n";   net_addr = NET_ADDR_CAMERA; } 
		//c_COMBO3.GetLBText( idx, str );
	    c_EDIT2.SetWindowText(out_put);
		//CString Out;
		//Out.Format( "Drop List Selection => index %d\n%s", idx, str );
		//AfxMessageBox( Out ); 
}

char CDBGui2Dlg::waitSerialByte() {
	char recBuffer;
	char numBytesRead;
	do {
		numBytesRead = serial.ReadData(&recBuffer, 1);
	} while (numBytesRead == 0);
	return recBuffer;
}